/****************************************************************************************
 *	Name		:	vldemo.c
 *	Title		:	3D VideoLogic Logo
 *	Author		:	VideoLogic Limited
 *	Created		:	20/09/1996
 *
 *	Copyright 	:	1995 by VideoLogic Limited. All rights reserved.
 *					No part of this software, either material or conceptual 
 *					may be copied or distributed, transmitted, transcribed,
 *					stored in a retrieval system or translated into any 
 *					human or computer language in any form by any means,
 *					electronic, mechanical, manual or other-wise, or 
 *					disclosed to third parties without the express written
 *					permission of VideoLogic Limited, Unit 8, HomePark
 *					Industrial Estate, King's Langley, Hertfordshire,
 *					WD4 8LZ, U.K.
 *
 *	Description	:	Creates 3D scene. Animates the 3D scene
 *
 *	Platform	:	ANSI compatible
 *
 *	History		:
 *
 * $Log:	vldemo.c $
 * Revision 1.2  97/02/19  12:41:31  EDWARDFORDE
 * Fixed background lighting issue when camera moved.
 * 
 * Revision 1.1  96/11/15  09:35:04  EdwardForde
 * Initial revision
 * 
 * 
 ***************************************************************************************/
																					   
/****************************************************************************************
/*								INCLUDE FILES											*
/***************************************************************************************/

#include <windows.h>
#include <stdlib.h>
#include <math.h>
#include "sgl.h"
#include "resource.h"
#include "vldemo.h"
#include "frontend.h"

/****************************************************************************************
/*								LOCAL DEFINE DEFINITIONS 								*
/***************************************************************************************/


/* define the logo points
 */
#define A {0.0f, 37.5f}
#define B {12.0f,37.5f}
#define C {6.0f, 19.5f}
#define D {6.5f, 18.0f}
#define E {13.0f,37.5f}
#define F {25.0f,37.5f}
#define G {12.5f, 0.0f}
#define H {13.5f, 0.0f}
#define I {19.5f, 18.0f}
#define J {31.5f, 18.0f}
#define K {25.5f, 0.0f}

/* define some points inbetween */
/* some of the above            */
#define AB {6.0f,37.5f}
#define DE {11.7f,33.6f}
#define FG {18.75f,18.75f}
#define JK {28.5f,9.0f}
#define KH {19.5f,0.0f}

#define MAXPARTS 6 	  /*Number of sections the logo is split into */

/****************************************************************************************
/*								EXTERNAL GLOBAL VARIABLES								*
/***************************************************************************************/

extern	HWND		hWnd;

extern	GLOBALVARS	globalVars;
extern	sgl_bool	moving_mode;
extern	sgl_vector	XAxis;	
extern	sgl_vector	YAxis;	
extern	sgl_vector	ZAxis;
extern	sgl_bool	MoveCamera;
extern	sgl_vector	camera_pos;

/* Lights 1, 2, 3 and 4 positions.
 */
extern	sgl_vector	light_pos1;
extern	sgl_vector	light_pos2;
extern	sgl_vector	light_pos3;
extern	sgl_vector	light_pos4;

extern	int			*material_ptr;
extern	int			PARTS;

/****************************************************************************************
/*								EXTERNAL FUNCTION DEFINITIONS							*
/***************************************************************************************/

extern int	create_rectprism	(float x, float y, float z, sgl_bool bName, sgl_bool bTexture, sgl_bool bHidden);
extern int	create_prism		(float height, float radius, int num_sides, sgl_bool bName, 
								 sgl_bool bSmoothShade, sgl_bool bTexture);
extern int	create_extruded_poly(int numpoints, float point_list[][2], float depth,
								 sgl_bool bName, sgl_bool bSmoothShade, sgl_bool bTexture);
extern void	SetupCachedTexture	(sgl_bool cached);
extern void	MoveCameraPos	(int cameraTransform, sgl_vector camera_pos);
extern void	MoveLightPos	(int lightTransform, int tLightVolume, sgl_vector light_pos);

/****************************************************************************************
/*								DEFINED GLOBAL VARIABLES								*
/***************************************************************************************/

int		nDummyList, nInstance, nInstance1, nTextureList;

/****************************************************************************************
/*								LOCAL VARIABLES											*
/***************************************************************************************/

int		transform_names[MAXPARTS];
int		animation_position;
int		hLightVolume;
int		tLightVolume;

/* Define the points of the object. The object is generated by
 * MAXPARTS objects and each object can have a maximum of 5 points.
 *
 *							# points	 1	2  3  4	 5
 */
static	sgl_2d_vec points[MAXPARTS][5]= {
										{A ,AB,C		},
										{AB,B ,C		},
										{D ,DE,FG,G		},
										{DE,E ,F ,FG	},
										{H ,I ,J ,JK,KH	},
										{JK,K ,KH		}
										};
int		num_points[MAXPARTS] = {3,3,4,4,5,3};

/* center of each object.
 */
sgl_vector	centres[MAXPARTS];

sgl_vector	rotates1[MAXPARTS];
sgl_vector	rotates2[MAXPARTS];

sgl_vector	translate_dist[MAXPARTS];
float		translate_rate[MAXPARTS];
int			material[MAXPARTS];
int			pointCollision;

/****************************************************************************************
/*								LOCAL FUNCTION DEFINITIONS								*
/***************************************************************************************/

void		SetupVideoLogicLogoDemo 	(void);
void		RunVideoLogicLogoDemo   	(sgl_bool do_render);
void		CloseVideoLogicLogoDemo 	(void);
static void Randomise					(void);
static void	InitialiseParameters		(void);

/****************************************************************************************
/*								LOCAL FUNCTIONS											*
/***************************************************************************************/

/****************************************************************************************
/*
/*	Function Name	:	Randomise
/*	Inputs			:	-
/*	Outputs			:	-
/*	Returns			:	-
/*	Globals Used	:	-
/*	Description		:	Routine to initialise parameters for the annimation of the scene.
/*
/***************************************************************************************/

static void Randomise(void)
{
	int i, j;
	int rotation1_character;
	int transrate_character;
	int transdist_character;
	int rotation2_character;
	unsigned	long	seed;

	/* Randomly select the seed.
	 */
	seed = (long) rand();
	sgl_srand(seed);

	/* Set up the overal behaviour of the animation
	 */
	rotation1_character = (int) sgl_rand() % 5 + 2;
	rotation2_character = (int) sgl_rand() % 20 + 5;
 	transrate_character = (int) sgl_rand() % 10 + 3;
	transdist_character = (int) sgl_rand() % 30 + 20;
	
	/* initialise the movement factors
	 */
	for(i = 0; i < MAXPARTS; i ++)
	{
		for(j= 0; j < 3; j++)
			rotates1[i][j] = (float) (rand() % rotation1_character);

		for(j= 0; j < 3; j++)
			rotates2[i][j] = (float) (rand() % rotation2_character);

		translate_rate[i] = (float) (rand() % transrate_character);

		for(j= 0; j < 3; j++)
			translate_dist[i][j] = (float) (rand() % transdist_character);
	}

}


/****************************************************************************************
/*
/*	Function Name	:	SetupVideoLogicLogoDemo
/*	Inputs			:	-
/*	Outputs			:	-
/*	Returns			:	-
/*	Globals Used	:	globalVars, light_pos, camera_pos, centers
/*	Description		:	Routine to initialise and create the VLlogo demo. It creates
/*						everything to do with the scene. i.e. view, lighting, shading,
/*						tranparency, objects etc.
/***************************************************************************************/

void SetupVideoLogicLogoDemo()
{
	sgl_vector				pointPos, CenterPos = {0.0f, 0.0f, 0.0f};
	int						nInstancedMaterial, i, j;
	sgl_intermediate_map	processedMap;

	sgl_colour  cBlack = {0.0f, 0.0f, 0.0f};
	sgl_colour  cWhite = {1.00f, 1.00f, 1.00f};
	sgl_colour  cGrey  = {0.70f, 0.70f, 0.70f};
	sgl_colour  dkgrey = {0.20f, 0.20f, 0.20f};
	sgl_colour  dkblue = {0.15f, 0.25f, 0.60f};
	sgl_colour  ltblue = {0.53f, 0.81f, 0.92f};
	sgl_colour  cGreen = {0.10f, 0.40f, 0.20f};

	sgl_colour  colours[MAXPARTS] = { 	{0.72549f, 0.0f,     0.14118f},
										{0.72549f, 0.0f,     0.14118f},
										{0.0f, 	   0.50196f, 0.23922f},
										{0.0f, 	   0.50196f, 0.23922f},
										{0.0f,     0.55294f, 0.76471f},
										{0.0f,     0.55294f, 0.76471f}};
	
    /* VL Demo setup inital values
     */
	InitialiseParameters();
	animation_position		= 0;
	globalVars.demoNumber	= 1;

	/* create screen device 0 with 640x480 screen resolution
	 * and 24 bit colour and no double buffering            
	 */
	globalVars.screenDevice = sgl_create_screen_device(	globalVars.numberDevice,
													globalVars.xDevice,	
													globalVars.yDevice,	
													globalVars.modeDevice,	
													globalVars.doublebufferDevice);

	/* set the viewport for the opened device so that there
	 * is a 20 pixel border around the rendered image      
	 */ 
	globalVars.viewport = sgl_create_viewport(	globalVars.screenDevice,
											globalVars.leftViewport,
											globalVars.topViewport,
											globalVars.rightViewport,
											globalVars.bottomViewport,
											globalVars.leftCamera,
											globalVars.topCamera,
											globalVars.rightCamera,
											globalVars.bottomCamera);

	/* setup the quality values. List which doesn't preserve the state.
	 */
	globalVars.hQualList = sgl_create_list(TRUE,FALSE,FALSE);
	{
		sgl_qual_texturing(globalVars.texturingQual);
		sgl_qual_smooth_shading(globalVars.shadingQual);
		sgl_qual_generate_shadows(globalVars.shadowQual);
		sgl_qual_fog(globalVars.fogQual);
		sgl_qual_collision_detection(globalVars.collisionQual,FALSE);
	}
	sgl_to_parent();

	/* Create material instance.
	 */
	sgl_create_list(UNAMED_ITEM,TRUE,TRUE);
	{
		nInstancedMaterial = sgl_create_material(NAMED_ITEM,FALSE);

		sgl_set_ambient(cWhite);
		sgl_set_diffuse(cWhite);
		sgl_set_specular(cWhite,FALSE);	/* turn specular reflectivity off */
		sgl_set_texture_effect(TRUE,TRUE,FALSE,FALSE);
	}sgl_to_parent();

	/* Create list instance for rectangle lights.
	 */
	nInstance = sgl_create_list(NAMED_ITEM,TRUE,TRUE);
	{
		globalVars.convexPolydron[globalVars.convexPolydronCnt++] = 
			create_rectprism(1.0f,5.0f,1.0f, NAMED_ITEM, NO_TEXTURE,FALSE);
	}sgl_to_parent();

	/* Create list instance for second rectangle lights.
	 */
	nInstance1 = sgl_create_list(NAMED_ITEM,TRUE,TRUE);
	{
		globalVars.convexPolydron[globalVars.convexPolydronCnt++] = 
			create_rectprism(2.0f,10.0f,2.0f, NAMED_ITEM, NO_TEXTURE,FALSE);
	}sgl_to_parent();

	sgl_modify_list(SGL_DEFAULT_LIST, FALSE);

	/*************************/
	/* SETUP OBJECT TEXTURES */
	/*************************/

	/* Clear texture handles.
	 */
	for (i = 0 ; i < 6 ; i++)
		globalVars.textureHandle[i] = 0;

	globalVars.textureNumberUsed	= 3;           
	globalVars.textureNumber		= 0;

	/* Check the amount of free texture memory.
	 */
	globalVars.lFreeMem = sgl_get_free_texture_mem();

	/* Create Brick texture for horizontal and vertical planes.
	 */
	strcpy (globalVars.textureName[0],"brick.bmp");
	globalVars.textureMem[0]     = ConvertBMPtoSGL(globalVars.textureName[0],FALSE);
	globalVars.textureHandle[0]  = sgl_create_texture(	sgl_map_16bit_mm,sgl_map_128x128,FALSE,
													TRUE,&(globalVars.textureMem[0]),NULL);
	sgl_texture_size(&(globalVars.textureMem[0]));
	sgl_set_texture(globalVars.textureHandle[0],FALSE,TRUE,&(globalVars.textureMem[0]),NULL);

	/* Create strip texture for VideoLogic Logo Sections. Done using preprocessed texture.
	 */
	strcpy (globalVars.textureName[1],"stripe.bmp");
	globalVars.textureMem[1]     = ConvertBMPtoSGL(globalVars.textureName[1],FALSE);
	sgl_preprocess_texture(	sgl_map_16bit_mm, sgl_map_64x64, FALSE, TRUE,
						&(globalVars.textureMem[1]), NULL, &processedMap);
	globalVars.textureHandle[1]  = sgl_create_texture(0, 0, 0, 0, &processedMap, NULL);


/*	globalVars.textureHandle[1]  = sgl_create_texture(	sgl_map_16bit_mm, sgl_map_64x64, FALSE,
													TRUE,&(globalVars.textureMem[1]),NULL);
 */
	sgl_texture_size(&(globalVars.textureMem[1]));

	/* Create Sky texture for background plane.
	 */
	strcpy (globalVars.textureName[2],"blursky.bmp");
	globalVars.textureMem[2]     = ConvertBMPtoSGL(globalVars.textureName[2],FALSE);
	globalVars.textureHandle[2]  = sgl_create_texture(	sgl_map_16bit_mm,sgl_map_256x256,FALSE,
													TRUE,&(globalVars.textureMem[2]),NULL);

	/****************/
	/* SETUP LIGHTS */
	/****************/

	globalVars.lightNumberUsed			   =  4;	/* number of lights.	*/
	globalVars.lightNumber                 =  0;	/* light being used.	*/

	/* set the values for the first light  (parallel light)
	 */
	globalVars.lightType            [0]    =  0;
	globalVars.lightOff             [0]    =  FALSE;
	globalVars.lightColour          [0][0] =  1.0f;
	globalVars.lightColour          [0][1] =  1.0f;
	globalVars.lightColour          [0][2] =  1.0f;
	globalVars.lightColoured        [0]    =  COLOUR;
	globalVars.lightDirection       [0][0] =  0.0f;
	globalVars.lightDirection       [0][1] = -0.5f;
	globalVars.lightDirection       [0][2] =  0.0f;
 	globalVars.lightPosition        [0][0] =  0.0f;
 	globalVars.lightPosition        [0][1] =  1.0f;
 	globalVars.lightPosition        [0][2] =  0.0f;
	globalVars.lightConcentration   [0]    =  0;
	globalVars.lightCastsShadows    [0]    =  NO_SHADOWS;
	globalVars.lightSmoothHighlights[0]    =  FALSE;

    globalVars.lightHandle[0] = sgl_create_parallel_light(NAMED_ITEM,
													globalVars.lightColour[0],
												  	globalVars.lightDirection[0],
													globalVars.lightCastsShadows[0],
													globalVars.lightSmoothHighlights[0]);

	/* set the values for the second light (parallel light)
	 */
	globalVars.lightType            [1]    =  0;
	globalVars.lightOff             [1]    =  FALSE;
	globalVars.lightColour          [1][0] =  1.0f;
	globalVars.lightColour          [1][1] =  1.0f;
	globalVars.lightColour          [1][2] =  1.0f;
	globalVars.lightColoured        [1]    =  COLOUR;
	globalVars.lightDirection       [1][0] =  0.0f;
	globalVars.lightDirection       [1][1] = -0.5f;
	globalVars.lightDirection       [1][2] =  0.0f;
 	globalVars.lightPosition        [1][0] =  0.0f;
 	globalVars.lightPosition        [1][1] =  1.0f;
 	globalVars.lightPosition        [1][2] =  0.0f;
	globalVars.lightConcentration   [1]    =  0;
	globalVars.lightCastsShadows    [1]    =  NO_SHADOWS;
	globalVars.lightSmoothHighlights[1]    =  FALSE;

	globalVars.lightHandle[1] = sgl_create_parallel_light(NAMED_ITEM,
													globalVars.lightColour[1],
													globalVars.lightDirection[1],
													globalVars.lightCastsShadows[1],
													globalVars.lightSmoothHighlights[1]);

	/* set the values for the third light (point light)
	 */
	globalVars.lightType            [2]    =  1;
	globalVars.lightOff             [2]    =  FALSE;
	globalVars.lightColour          [2][0] =  1.0f;
	globalVars.lightColour          [2][1] =  1.0f;
	globalVars.lightColour          [2][2] =  1.0f;
	globalVars.lightColoured        [2]    =  COLOUR;
	globalVars.lightDirection       [2][0] =  0.0f;
	globalVars.lightDirection       [2][1] = -0.5f;
	globalVars.lightDirection       [2][2] =  0.0f;
 	globalVars.lightPosition        [2][0] =  0.0f;
 	globalVars.lightPosition        [2][1] =  100.0f;
 	globalVars.lightPosition        [2][2] =  0.0f;
	globalVars.lightConcentration   [2]    =  32;
	globalVars.lightCastsShadows    [2]    =  NO_SHADOWS;
	globalVars.lightSmoothHighlights[2]    =  FALSE;

	globalVars.lightHandle[2] = sgl_create_point_light(	NAMED_ITEM,
													globalVars.lightColour[2],
											 		globalVars.lightDirection[2],
								 					globalVars.lightPosition[2],
													globalVars.lightConcentration[2],
													globalVars.lightCastsShadows[2],
													globalVars.lightSmoothHighlights[2]);

	/* set the values for the fouth light (point light)
	 */
	globalVars.lightType            [3]    =  1;
	globalVars.lightOff             [3]    =  FALSE;
	globalVars.lightColour          [3][0] =  1.0f;
	globalVars.lightColour          [3][1] =  1.0f;
	globalVars.lightColour          [3][2] =  1.0f;
	globalVars.lightColoured        [3]    =  COLOUR;
	globalVars.lightDirection       [3][0] =  0.0f;
	globalVars.lightDirection       [3][1] = -0.5f;
	globalVars.lightDirection       [3][2] =  0.0f;
 	globalVars.lightPosition        [3][0] =  0.0f;
 	globalVars.lightPosition        [3][1] =  100.0f;
 	globalVars.lightPosition        [3][2] =  0.0f;
	globalVars.lightConcentration   [3]    =  32;
	globalVars.lightCastsShadows    [3]    =  NO_SHADOWS;
	globalVars.lightSmoothHighlights[3]    =  FALSE;

	globalVars.lightHandle[3] = sgl_create_point_light(	NAMED_ITEM,
													globalVars.lightColour[3],
								 					globalVars.lightDirection[3],
								 					globalVars.lightPosition[3],
													globalVars.lightConcentration[3],
													globalVars.lightCastsShadows[3],
													globalVars.lightSmoothHighlights[3]);

	/* Switch shadows OFF so that light prisms are not shadowed.
	 */
	sgl_switch_light( globalVars.lightHandle[0],TRUE,NO_SHADOWS,FALSE); 
	sgl_switch_light( globalVars.lightHandle[1],TRUE,NO_SHADOWS,FALSE); 
	sgl_switch_light( globalVars.lightHandle[2],TRUE,NO_SHADOWS,FALSE); 
	sgl_switch_light( globalVars.lightHandle[3],TRUE,NO_SHADOWS,FALSE); 

	globalVars.lightList = sgl_create_list(NAMED_ITEM,FALSE,FALSE);
	{
		/* Dummy List to insert substitution.
		 */
	}
	sgl_to_parent();

	/* Creates physical light object 1.
	 */
	sgl_create_list(UNAMED_ITEM,TRUE,FALSE);
	{
		globalVars.lightTransform[0] = sgl_create_transform(NAMED_ITEM);

		sgl_position_light (globalVars.lightHandle[0]);

		sgl_translate(0.0f, 50.0f,  0.0f);
		sgl_use_instance(nInstance);
	}
	sgl_to_parent();

	/* Creates physical light object 2.
	 */
 	sgl_create_list(UNAMED_ITEM,TRUE,FALSE);
	{
		globalVars.lightTransform[1] = sgl_create_transform(NAMED_ITEM);
		sgl_position_light (globalVars.lightHandle[1]);
		sgl_translate(0.0f, 50.0f,  0.0f);
		sgl_use_instance(nInstance);
	}
	sgl_to_parent();
	
	/* Creates physical light object 3.
	 */
 	sgl_create_list(UNAMED_ITEM,TRUE,FALSE);
	{
		globalVars.lightTransform[2] = sgl_create_transform(NAMED_ITEM);
		sgl_position_light (globalVars.lightHandle[2]);
		sgl_translate(0.0f, 50.0f,  0.0f);
		sgl_use_instance(nInstance);
	}
	sgl_to_parent();
	 
	/* Creates physical light object 4.
	 */
 	sgl_create_list(UNAMED_ITEM,TRUE,FALSE);
	{
		globalVars.lightTransform[3] = sgl_create_transform(NAMED_ITEM);
		sgl_position_light (globalVars.lightHandle[3]);
		sgl_translate(0.0f, 50.0f,  0.0f);
		sgl_use_instance(nInstance);
	}
	sgl_to_parent();

	/* Switch shadows ON so that light prisms are not shadowed.
	 */
	sgl_switch_light( globalVars.lightHandle[0],TRUE,SHADOW_LIGHT,FALSE); 
	sgl_switch_light( globalVars.lightHandle[1],TRUE,SHADOW_LIGHT,FALSE); 
	sgl_switch_light( globalVars.lightHandle[2],TRUE,SHADOW_LIGHT,FALSE); 
	sgl_switch_light( globalVars.lightHandle[3],TRUE,SHADOW_LIGHT,FALSE); 


	/**********************************/
	/* SETUP LIGHT VOLUME FOR LIGHT 4 */
	/**********************************/	

	hLightVolume = sgl_create_list(TRUE,TRUE,TRUE);
	{
		sgl_create_light_volume(NAMED_ITEM,TRUE,globalVars.lightHandle[3]);
		{
			sgl_vector cubepoint[6] = {	{ 3.0f,   0.0f, 0.0f},
										{ 0.0f, 100.0f, 0.0f},
										{ 0.0f,   0.0f, 3.0f},
										{-3.0f,   0.0f, 0.0f},
										{ 0.0f,-100.0f, 0.0f},
										{ 0.0f,   0.0f,-3.0f}};
		
			sgl_add_simple_plane(cubepoint[0],cubepoint[0],FALSE);
			sgl_add_simple_plane(cubepoint[1],cubepoint[1],FALSE);
			sgl_add_simple_plane(cubepoint[2],cubepoint[2],FALSE);
			sgl_add_simple_plane(cubepoint[3],cubepoint[3],FALSE);
			sgl_add_simple_plane(cubepoint[4],cubepoint[4],FALSE);
			sgl_add_simple_plane(cubepoint[5],cubepoint[5],FALSE);
		}
	}
	sgl_modify_list(SGL_DEFAULT_LIST,FALSE);

	sgl_create_list(UNAMED_ITEM,TRUE,FALSE);
	{
		int i,j;

		sgl_translate(-50.0f,0.0f,-50.0f);

		tLightVolume = sgl_create_transform(NAMED_ITEM);

		for (i = 0; i < 10; i++)
		{
			for (j = 0; j < 10; j++)
			{
				sgl_use_instance(hLightVolume);
				sgl_translate(10.0f, 0.0f, 0.0f);
			}
			sgl_translate(-100.0f, 0.0f, 10.0f);
		}
	}
	sgl_to_parent();

	/********************/
	/* PLACE THE CAMERA */
	/********************/
	globalVars.cameraList = sgl_create_list(NAMED_ITEM,TRUE,FALSE);
  	{
		/* Setup camera.
		 */
		globalVars.cameraTransform = sgl_create_transform(NAMED_ITEM);
		sgl_translate(0.0f, 0.0f,  -68.0f);
		globalVars.camera = sgl_create_camera(	globalVars.zoomCamera,
												globalVars.foregroundCamera,
												globalVars.inv_backgroundCamera);

		globalVars.backgroundList = sgl_create_list(NAMED_ITEM,TRUE,FALSE);
		{
			/* Used to create the background image. i.e. the sky.
			 * Create here so that it is always visible on the camera.
			 * Three points are required to create a plane.
			 */
			float		uv1[2], uv2[2] , uv3[2];
			sgl_vector	BackPnt		= { 0.0f,0.0f, 10000.0f };	
			sgl_vector	Backpoint2	= { 0.0f,10.0f, 10000.0f };	
			sgl_vector	Backpoint3	= { 10.0f, 0.0f, 10000.0f };

			uv1[0] = 0.0f;
			uv1[1] = 0.0f;

			uv2[0] = 0.0f;
			uv2[1] = 0.002f;

			uv3[0] = 0.002f;
			uv3[1] = 0.0f;

			sgl_set_diffuse(cBlack);
			sgl_set_glow(ltblue);
			sgl_set_ambient(cWhite);

			globalVars.textureList[2] = sgl_create_list(TRUE,FALSE,FALSE);
			{
				sgl_set_texture_effect(FALSE,TRUE,FALSE,TRUE);
				sgl_set_texture_map(globalVars.textureHandle[2],FALSE,FALSE);
			}
			sgl_to_parent();
			sgl_add_plane(	BackPnt, Backpoint2, Backpoint3, VISIBLE,
						NULL,NULL,NULL,uv1,uv2,uv3);
		}
		sgl_to_parent();
	}
	sgl_to_parent();

	/*************/
	/* SETUP FOG */
	/*************/

	globalVars.fogColour[0] = 0.0f;
	globalVars.fogColour[1] = 0.0f;
	globalVars.fogColour[2] = 0.0f;
	globalVars.fogDensity   = 0.0f;

	sgl_set_fog(globalVars.camera, globalVars.fogColour, globalVars.fogDensity);

	/**********************/
	/* SETUP MULTISHADOWS */
	/**********************/
	/* As of the 7/10/96 not operational.
	 *
	sgl_pseudo_multishadows(globalVars.bMultiShadows, globalVars.MultiShadowsColour);
	 */

	/*****************/
	/* SETUP AMBIENT */
	/*****************/

	globalVars.ambientHandle = sgl_create_ambient_light(	NAMED_ITEM, globalVars.ambientColour, FALSE);

	/***************************/
	/* CREATE VIDEOLOGIC LOGO  */
	/***************************/

	sgl_create_list(UNAMED_ITEM,TRUE,FALSE);
	{
		/* Create the texture on the logo.
		 */
		globalVars.nTransform = sgl_create_transform(NAMED_ITEM);

		/* Dummy List to test Detach/Attach list.
		 */
		nDummyList = sgl_create_list(NAMED_ITEM,FALSE,FALSE);
		{
			globalVars.textureList[1] = sgl_create_list(TRUE,FALSE,FALSE);
			{
				nTextureList = globalVars.textureList[1];

				globalVars.materialHandle[1] = sgl_create_material(NAMED_ITEM,FALSE);

				sgl_set_ambient		(globalVars.ambientMatColour);
				sgl_set_specular	(globalVars.specularMatColour,globalVars.specularMatShine);
				sgl_set_glow		(globalVars.glowMatColour);
				sgl_set_texture_effect(TRUE,TRUE,FALSE,FALSE);

				/* Get strip texture.
				 */
				sgl_set_texture_map(globalVars.textureHandle[1],FALSE,FALSE);
			}
			sgl_to_parent();
		}
		sgl_to_parent();

		/* Move logo to center on screen. Moves center of logo to the origin.
		 */
		sgl_translate(-15.75f,  -18.75f, 0.0f);

		/* Create collision point. Creates a collision point at 0,0,0 and then
		 * resets the collision point to the top left hande corner of the logo.
		 */
		pointCollision = sgl_create_point(CenterPos, TRUE);

		pointPos[0] = points[0][0][0];
		pointPos[1] = points[0][0][1];
		pointPos[2] = 0.0f;
	
		sgl_set_point(pointCollision, pointPos, TRUE);

		/* create each of the objects. creates each part seperately.
		 */
		for(i = 0; i < MAXPARTS; i++)
		{		
			sgl_create_list(UNAMED_ITEM,TRUE,FALSE);
			{
				/* create Null positioning info
				 */	
				transform_names[i] = sgl_create_transform(NAMED_ITEM);
				material[i]        = sgl_create_material(NAMED_ITEM,FALSE);

				/* set the colour.
				 */
				sgl_set_diffuse(colours[i]);

				/* select every second object.
				 */
				if(i & 1)
				{
					sgl_set_opacity(globalVars.opacityMatValue);
					sgl_new_translucent();
				}
         
				/* create the 3D logo.
				 */
				globalVars.convexPolydron[globalVars.convexPolydronCnt++] = 
						create_extruded_poly(	num_points[i], points[i], DEPTH, NAMED_ITEM,
												!SMOOTH_SHADING, TEXTURE);

				if (i == 0)
				{
					/* Position collision point. Positioned here so that it rotates
					 * with the logo section.
					 */
					sgl_position_point(pointCollision);

					/* Switch collision detection OFF.
					 */
					sgl_switch_point(pointCollision, FALSE);
				}
			}
			sgl_to_parent();
		}

		/* Create invisible rectprism which is used for collision detection.
		 * A collision occurs when the logo returns to its fixed state.
		 */
		sgl_create_list(UNAMED_ITEM,TRUE,FALSE);
		{
			/* Switch collision detection ON.
			 */
			sgl_switch_point(pointCollision, TRUE);

			sgl_translate(pointPos[0], pointPos[1], pointPos[2]);
			create_rectprism(0.1f,0.1f,0.1f, UNAMED_ITEM, NO_TEXTURE,TRUE);

			/* Switch collision detection OFF.
			 */
			sgl_switch_point(pointCollision, FALSE);
		}
		sgl_to_parent();

	}
	sgl_to_parent();

	/************************/
	/* CREATE BRICK PLANES  */
	/************************/

	sgl_create_list(UNAMED_ITEM,TRUE,FALSE);
	{
		globalVars.textureList[0] = sgl_create_list(TRUE,FALSE,FALSE);
		{
			/* Use an instance to a material.
			 */
			sgl_use_material_instance(nInstancedMaterial, FALSE);

			/* Copy instance to material handle.
			 */
			globalVars.materialHandle[0] = nInstancedMaterial;
	
			/* Get brick texture.
			 */
			sgl_set_texture_map(globalVars.textureHandle[0],FALSE,FALSE);
		}
		sgl_to_parent();

		/* create the two brick planes.
		 */
		sgl_translate(0.0f, -30.0f, 0.0f);
		sgl_set_diffuse(cGrey);
		globalVars.horizPlanePolydron =
			create_prism(3.0f, RADIUS, 16, NAMED_ITEM, 1, TEXTURE);
		globalVars.convexPolydron[globalVars.convexPolydronCnt++] = globalVars.horizPlanePolydron;

		sgl_translate(0.0f, +30.0f, 60.0f);
		sgl_rotate(sgl_x_axis, PI/2.0f);
		sgl_rotate(sgl_y_axis, PI/2.0f);
		globalVars.vertPlanePolydron = create_prism(3.0f, RADIUS, 16, NAMED_ITEM, 1, TEXTURE);
		globalVars.convexPolydron[globalVars.convexPolydronCnt++] = globalVars.vertPlanePolydron;
	}
	sgl_to_parent();

	/* initialise the movement factors
	 */
	Randomise();

	/* Set up the centres of the objects
	 */
	for(i = 0; i < MAXPARTS; i++)
	{
		/* Initialise values.
		 */
		centres[i][0]=0.0F;
		centres[i][1]=0.0F;
		centres[i][2]=DEPTH/2.0F;

		/* Get centers for each object.
		 * Add all the points together.
		 */
		for(j = 0; j < num_points[i]; j++)
		{
			centres[i][0] +=points[i][j][0];
			centres[i][1] +=points[i][j][1];
		}

		/* Divide by the number of points to get the
		 * center point.
		 */
		centres[i][0] /= num_points[i];
		centres[i][1] /= num_points[i];
	}


	/* Initialisation of camera and light positions.
	 */
	camera_pos[0] = 0.0f;
	camera_pos[1] = 0.0f;
	camera_pos[2] = 2.0f;

	light_pos1[0] = -0.18f;
	light_pos1[1] = -0.2f;
	light_pos1[2] = 0.0f;

	light_pos2[0] = -0.3333f;
	light_pos2[1] = -0.3333f;
	light_pos2[2] = 0.0f;

	light_pos3[0] = 0.0f;
	light_pos3[1] = -0.3333f;
	light_pos3[2] = 0.0f;

	light_pos4[0] = 0.3333f;
	light_pos4[1] = -0.3333f;
	light_pos4[2] = 0.0f;

	/* Setup camera position.
	 */
	sgl_modify_transform(globalVars.cameraTransform, TRUE);
	sgl_rotate(YAxis, -camera_pos[0] * 4.712385f);
	sgl_rotate(XAxis, -camera_pos[1] * 4.712385f);
	sgl_translate(0.0f, 0.0f,  -60.0f * camera_pos[2]);

	/* Set up the light 1 position.
	 */
	sgl_modify_transform(globalVars.lightTransform[0], TRUE);
	sgl_rotate(YAxis, light_pos1[0] * 4.712385f);
	sgl_rotate(XAxis, light_pos1[1] * 4.712385f);

	/* Set up the light 2 position.
	 */
	sgl_modify_transform(globalVars.lightTransform[1], TRUE);
	sgl_rotate(YAxis, light_pos2[0] * 4.712385f);
	sgl_rotate(XAxis, light_pos2[1] * 4.712385f);

	/* Set up the light 3 position.
	 */
	sgl_modify_transform(globalVars.lightTransform[2], TRUE);
	sgl_rotate(YAxis, light_pos3[0] * 4.712385f);
	sgl_rotate(XAxis, light_pos3[1] * 4.712385f);

	/* Set up the light 4 position.
	 */
	sgl_modify_transform(globalVars.lightTransform[3], TRUE);
	sgl_rotate(YAxis, light_pos4[0] * 4.712385f);
	sgl_rotate(XAxis, light_pos4[1] * 4.712385f);

	/* Initialise material pointer and number of parts variable.
	 */
	material_ptr	= material;
	PARTS			= MAXPARTS;
}


/****************************************************************************************
/*
/*	Function Name	:	RunVideoLogicLogoDemo
/*	Inputs			:	sgl_bool	do_render
/*
/*	Outputs			:	Renders VideoLogic Logo to display.
/*	Returns			:	-
/*	Globals Used	:	globalVars, light_pos, camera_pos, moving_mode, centers
/*						animation_position.
/*	Description		:	Routine which controls the annimation of the VLlogo scene.
/*						Renders the image to the scene.
/*
/***************************************************************************************/

void RunVideoLogicLogoDemo(	sgl_bool do_render)
{
	float				fraction;
	static int			nBMPNo;
	int					i;
	sgl_collision_data	collisionData;
	sgl_vector			pointPos;
	static BOOL			collision = FALSE;

	/* set up the camera position. Changed by MOUSE.
	 * Calculations only performed if camera selected.
	 *
	 *	3.14159f * 1.5f = 4.712385f
	 */
	if (MoveCamera)
		MoveCameraPos(globalVars.cameraTransform, camera_pos);
	else
	{
		/* Only perform light position calculations for light enabled.
		 */
		switch (globalVars.lightMove)
		{
			case 1	:
					/* Set up the light 1 position. Changed by MOUSE.
					 */
					MoveLightPos(globalVars.lightTransform[0], 0, light_pos1);
			break;						

			case 2	:
					/* Set up the light 2 position. Changed by MOUSE.
					 */
					MoveLightPos(globalVars.lightTransform[1], 0, light_pos2);
			break;

			case 3	:
					/* Set up the light 3 position. Changed by MOUSE.
					 */
					MoveLightPos(globalVars.lightTransform[2], 0, light_pos3);
			break;

			case 4	:
					/* Set up the light 4 position. Changed by MOUSE.
					 */
					MoveLightPos(globalVars.lightTransform[3], tLightVolume, light_pos4);
			break;

			default:
					/* do nothing...
					 */
			break;
		}
	}

	/* do the animation
	 */
	if(moving_mode)
	{
		fraction = (float) animation_position / (float) MOVE_CYCLES;

		/* causes acceleration/deceleration
		 */
		fraction = 0.5f + (float) (cos(fraction * 3.1416f) / 2.0f);

		/* set up the next frame. Take each part of the object in turn.
		 */
		for(i = 0; i < MAXPARTS; i ++)
		{
			sgl_modify_transform(transform_names[i], TRUE);

			/* translate origin to centre of object
			 */
			sgl_translate(centres[i][0], centres[i][1], centres[i][2]);

			/* do first lot of rotations
			 */
			sgl_rotate(XAxis, rotates1[i][0] * fraction * TWOPI);
			sgl_rotate(YAxis, rotates1[i][1] * fraction * TWOPI);
			sgl_rotate(ZAxis, rotates1[i][2] * fraction * TWOPI);

			/* translate back again
			 */
			sgl_translate(-centres[i][0], -centres[i][1],- centres[i][2]);

			/* Do some translations
			 */
			sgl_translate(	(float) (translate_dist[i][0] * sin(fraction * PI * translate_rate[i])),
							(float) (translate_dist[i][1] * sin(fraction * PI * translate_rate[i])),
							(float) (translate_dist[i][2] * sin(fraction * PI * translate_rate[i])));
				
			/* translate to centre of object
			 */
			sgl_translate(centres[i][0], centres[i][1], centres[i][2]);

			/* do some more rotations
			 */
		 	sgl_rotate(XAxis, rotates2[i][0] * fraction * TWOPI);
			sgl_rotate(YAxis, rotates2[i][1] * fraction * TWOPI);
			sgl_rotate(ZAxis, rotates2[i][2] * fraction * TWOPI);		

			/* translate back again
			 */
			sgl_translate(-centres[i][0], -centres[i][1], -centres[i][2]);
		}
				
		/* change the animation mode if necessary
		 */
		animation_position++;
	}
    
	/* Do Render
	 */
	if (do_render)
	{
		/* Do Render
		 */
		sgl_render(globalVars.viewport, globalVars.camera, FALSE);

		/* Check for collisions.
		 * As of 4/10/96 the is a problem with collision and release version.
		 */
		sgl_query_point(pointCollision, pointPos, &collisionData);
		if (collisionData.collision)
		{
			if (!collision)
			{
				/* for debug purposes.
				 */
	//			moving_mode = FALSE;

		        AppMsgBox(hWnd, MB_OK | MB_ICONEXCLAMATION,
        	          TEXT("Collision Occured"));
			}
			collision = TRUE;
		}
		else
			collision = FALSE;
	}
}


/****************************************************************************************
/*
/*	Function Name	:	CloseVideoLogicLogoDemo
/*	Inputs			:	-
/*	Outputs			:	Deletes textures, device and viewport.
/*	Returns			:	-
/*	Globals Used	:	globalVars
/*	Description		:	Routine to close down the VLlogo demo. Deletes the set device and
/*						releases the defined list.
/***************************************************************************************/

void CloseVideoLogicLogoDemo (void)
{
	int	i;

	/* Delete Cached Textures.
	 */
	SetupCachedTexture(FALSE);

	/* Delete Textures.
	 */
	for (i = 0 ; i < 6 ; i++)
	{
		if (globalVars.textureHandle[i])
		{
			sgl_delete_texture(globalVars.textureHandle[i]);
			globalVars.textureHandle[i] = 0;
		}
	}

	sgl_delete_viewport(globalVars.viewport);
	sgl_delete_device(globalVars.screenDevice);

	if (globalVars.nCachedTexture)
	{
		/* Delete cached texturing.
		 */
		sgl_register_texture_callback(NULL,globalVars.texData,1);

		sgl_unload_cached_texture(globalVars.nCachedTexture);
		globalVars.nCachedTexture = 0;
	}

	/* Clear all convexes.
	 */
	for (i = 0 ; i < globalVars.convexPolydronCnt ; i++)
	{
		if (globalVars.convexPolydron[i])
		{
			sgl_modify_convex(globalVars.convexPolydron[i], TRUE);
			globalVars.convexPolydron[i] = 0;
		}
	}

	sgl_modify_list(SGL_DEFAULT_LIST,TRUE);
}


/****************************************************************************************
/*
/*	Function Name	:	InitialiseParameters
/*	Inputs			:	sgl_bool	do_render
/*
/*	Outputs			:	-
/*	Returns			:	-
/*	Globals Used	:	globalVars
/*	Description		:	Initialises different parameters and settings.
/*
/***************************************************************************************/

static void InitialiseParameters()
{
	/* initialise multishadows settings.
	 */
	globalVars.bMultiShadows = TRUE;
	globalVars.MultiShadowsColour[0] = 0.1f;
	globalVars.MultiShadowsColour[1] = 0.1f;
	globalVars.MultiShadowsColour[2] = 0.1f;

	/* initialise convex handle count to zero.
	 */
	globalVars.convexPolydronCnt = 0;

	globalVars.ambientColour[0] = 0.0f;
	globalVars.ambientColour[1] = 0.0f;
	globalVars.ambientColour[2] = 0.0f;

	/* Initialise material menu options.
	 */
	globalVars.diffuseMatColour[0] = 0.0f;
	globalVars.diffuseMatColour[1] = 0.0f;
	globalVars.diffuseMatColour[2] = 0.0f;

	globalVars.ambientMatColour[0] = 0.0f;
	globalVars.ambientMatColour[1] = 0.0f;
	globalVars.ambientMatColour[2] = 0.0f;

	globalVars.specularMatColour[0] = 0.0f;
	globalVars.specularMatColour[1] = 0.0f;
	globalVars.specularMatColour[2] = 0.0f;
	globalVars.specularMatShine = 0;

	globalVars.glowMatColour[0] = 0.0f;
	globalVars.glowMatColour[1] = 0.0f;
	globalVars.glowMatColour[2] = 0.0f;

	globalVars.opacityMatValue = 0.5f;

	/* Initialise quality menu options.
	 */
	globalVars.texturingQual	= TRUE;
	globalVars.shadingQual		= FALSE;
	globalVars.shadowQual		= TRUE;
	globalVars.fogQual			= TRUE;
	globalVars.collisionQual	= FALSE;

 	/* Initialise background colour.
	 */
	globalVars.backgroundColour[0] = 0.0f;
	globalVars.backgroundColour[1] = 0.0f;
	globalVars.backgroundColour[2] = 0.0f;

 	/* Initialise camera and viewport settings.
	 */
	globalVars.zoomCamera			= LENS_35MM;
	globalVars.foregroundCamera		= 10.0f;
	globalVars.inv_backgroundCamera	= 0.0f;

	globalVars.leftViewport		= 0;
	globalVars.topViewport		= 0;
	globalVars.rightViewport	= 640;
	globalVars.bottomViewport	= 480;
	globalVars.leftCamera		= 80.0f;
	globalVars.topCamera		= 0.0f;
	globalVars.rightCamera		= 560.0f;
	globalVars.bottomCamera		= 480.0f;

	/* Setup device settings.
	 */
	globalVars.numberDevice		= 0;
	globalVars.xDevice			= 640;
	globalVars.yDevice			= 480;
	globalVars.modeDevice		= sgl_device_16bit;
	globalVars.doublebufferDevice = TRUE;
}
